@@ -1,3 +1,27 @@
+/*
+ * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
+ *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
+ * Reserved.  This file contains Original Code and/or Modifications of
+ * Original Code as defined in and that are subject to the Apple Public
+ * Source License Version 1.0 (the 'License').  You may not use this file
+ * except in compliance with the License.  Please obtain a copy of the
+ * License at http://www.apple.com/publicsource and read it before using
+ * this file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License."
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
+
 #import <sys/types.h>
 #import <stdio.h>
 #import <netinet/in.h>
@@ -16,18 +40,9 @@ void print_string(char *s, FILE *out)
 	int i, len;
 
 	len = strlen(s);
-	for (i = 0; i < len; i++) {
+	for (i = 0; i < len; i++)
+	{
 		if (s[i] == '"') fprintf(out, "\\\"");
-		else if (s[i] == '\a') fprintf(out, "\\a");
-		else if (s[i] == '\b') fprintf(out, "\\b");
-		else if (s[i] == '\f') fprintf(out, "\\f");
-		else if (s[i] == '\n') fprintf(out, "\\n");
-		else if (s[i] == '\r') fprintf(out, "\\r");
-		else if (s[i] == '\t') fprintf(out, "\\t");
-		else if (s[i] == '\v') fprintf(out, "\\v");
-		else if (s[i] == '\\') fprintf(out, "\\");
-		else if (s[i] < 20) fprintf(out, "\\0%0o", s[i]);
-		else if (s[i] == 0x7f) fprintf(out, "\377");
 		else putc(s[i], out);
 	}
 }
